#include "testApp.h"



int letter = 'm';
int letter2 = 't';


//--------------------------------------------------------------
void testApp::setup(){
	ofBackground(240,240,240);

	testFont.loadFont("Trebuchet.ttf", 300, true, true, true);

	ofSetFullscreen(true);
	ofGetFrameNum() % 190;
	
	
	panel.setup("control", 770, 0, 300, 400);
	panel.addPanel("flocking", 1, false);
	panel.setWhichPanel("flocking");
	panel.setWhichColumn(0);
	panel.addSlider("cohesion strength", "COHESION_STRENGTH", 0.01,0,1, false);
	panel.addSlider("cohesion radius", "COHESION_RADIUS", 50,0,200, false);
	panel.addSlider("alignment strength", "ALIGNMENT_STRENGTH", 0.01,0,1, false);
	panel.addSlider("alignment radius", "ALIGNMENT_RADIUS",  50,0,200, false);
	panel.addSlider("seperation strength", "SEPERATION_STRENGTH", 0.01,0, 1, false);
	panel.addSlider("seperation radius", "SEPERATION_RADIUS",  50,0,200, false);
	panel.addSlider("damping", "DAMPING", 0.01, 0.001, 0.5, false);
	panel.loadSettings("settings.xml");
	
	
	testChar = testFont.getCharacterAsPoints(letter);
	testChar2 = testFont.getCharacterAsPoints(letter2);
	
	
	
	resampler.clear();		/// CLEAR THE RESAMPLER
	
	if (testChar.contours.size() > 0){
		
		// ADD POINTS FROM THE CONTOUR TO THE RESAMPLER
		for(int i = 0; i < testChar.contours[0].pts.size(); i++){
			resampler.addPoint(testChar.contours[0].pts[i].x + 300, testChar.contours[0].pts[i].y + 300);
		}
		
		// RESAMPLE 
		resampler.resample(100);
		
		// ADD PARTICLES
		for (int i  = 0; i < 100; i++){
			particle tempParticle;
			tempParticle.setInitialCondition(resampler.pts[i].x, resampler.pts[i].y,0,0);
			Mpts.push_back(tempParticle);
		}
	}
	
	
	resampler2.clear();
	
	if (testChar2.contours.size() > 0){
		
		// ADD POINTS FROM THE CONTOUR TO THE RESAMPLER
		for(int i = 0; i < testChar2.contours[0].pts.size(); i++){
			resampler2.addPoint(testChar2.contours[0].pts[i].x + 300, testChar2.contours[0].pts[i].y + 300);
		}
		
		// RESAMPLE 
		resampler2.resample(100);
		
	}
	
	}

//--------------------------------------------------------------
void testApp::update(){

	printf("size of particles array = %i \n", (int)Mpts.size());
	
	panel.update();
	
	for (int i = 0; i < Mpts.size(); i++){
		Mpts[i].cohesion.distance = panel.getValueF("COHESION_RADIUS");
		Mpts[i].alignment.distance = panel.getValueF("ALIGNMENT_RADIUS");
		Mpts[i].seperation.distance = panel.getValueF("SEPERATION_RADIUS");
		
		
		Mpts[i].cohesion.strength = panel.getValueF("COHESION_STRENGTH");
		Mpts[i].alignment.strength = panel.getValueF("ALIGNMENT_STRENGTH");
		Mpts[i].seperation.strength = panel.getValueF("SEPERATION_STRENGTH");
		
		Mpts[i].damping = panel.getValueF("DAMPING");
		

	
	// on every frame 
	// we reset the forces
	// add in any forces on the particle
	// perfom damping and
	// then update
	
	
	for (int i = 0; i < Mpts.size(); i++){
		Mpts[i].resetForce();
		Mpts[i].damping = 0.1;
	}
		
	for (int i = 0; i < Mpts.size(); i++){

		Mpts[i].addAttractionForce(resampler2.pts[i].x, resampler2.pts[i].y, 1000, 0.07);
		
		for (int j = 0; j < i; j++){
			Mpts[i].addRepulsionForce(Mpts[j], 10, 1.5);
			Mpts[i].addRepulsionForce(mouseX, mouseY, 350, 0.05);
			}
		Mpts[i].addDampingForce();
		Mpts[i].update();
			
	}
	}
	
	
	ofDisableAlphaBlending();


}

//--------------------------------------------------------------
void testApp::draw(){

	ofSetColor(0, 90, 60);
	ofNoFill();
	
	//Start to draw first character (m)
				
	ofSetColor(255,0,0);
	for (int i = 0; i < Mpts.size(); i++){
		Mpts[i].draw();
	}
		
		panel.draw();	
	//Start to draw second character (t)
	
	float alpha,dist,distcenter = 0;
	for (int i = 0; i < Mpts.size(); i++)
	{
		Mpts[i].draw();
		
		for(int j = 0; j < i; j++)
		{	
			dist = ofDist(Mpts[i].pos.x, Mpts[i].pos.y,Mpts[j].pos.x, Mpts[j].pos.y);
			distcenter = ofDist(Mpts[j].pos.x, Mpts[j].pos.y,ofGetWidth()/2,ofGetHeight()/2);
			if(dist < 600)
			{
				alpha = ofMap(dist, 0, 70, 0, 35);
				ofSetColor(0, 0, 80, 65-alpha);
				ofNoFill();
				ofBeginShape();
				ofVertex(Mpts[i].pos.x, Mpts[i].pos.y);
				ofVertex(Mpts[j].pos.x, Mpts[j].pos.y);
				ofEndShape(false);
			}
		}
	}
		

				
}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){
	letter = key;
	
	testChar2 = testFont.getCharacterAsPoints(letter);
	
	
	resampler2.clear();
	
	if (testChar2.contours.size() > 0){
		
		// ADD POINTS FROM THE CONTOUR TO THE RESAMPLER
		for(int i = 0; i < testChar2.contours[0].pts.size(); i++){
			resampler2.addPoint(testChar2.contours[0].pts[i].x + 450, testChar2.contours[0].pts[i].y + 500);
		}
		
		resampler2.addPoint(testChar2.contours[0].pts[0].x + 450, testChar2.contours[0].pts[0].y + 500);
		
		// RESAMPLE 
		resampler2.resample(100);
		
	}
	
}

//--------------------------------------------------------------
void testApp::keyReleased  (int key){
	
	switch (key){
			
		case ' ':
			// reposition everything: 
			for (int i = 0; i < Mpts.size(); i++){
				Mpts[i].setInitialCondition(ofRandom(0,ofGetWidth()),ofRandom(0,ofGetHeight()),0,0);
			}
			break;
	}
	
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	
	if (x < 500){
		Mpts.erase(Mpts.begin());
		particle myParticle;
		float randomAngle = ofRandom(0, TWO_PI);
		myParticle.setInitialCondition(x,y,cos(randomAngle),sin(randomAngle));
		Mpts.push_back(myParticle);
	}
	
	panel.mouseDragged(x,y,button);
	
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	
	panel.mousePressed(x,y,button);
	
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	
	panel.mouseReleased();

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}
